import requests



class RestClient:

    def __init__(self, baseurl: str, token: str | None = None):
        self.baseurl = baseurl
        self.token = token

    def _headers(self) -> dict:
        if self.token:
            return {"Authorization": f"Token {self.token}"}
        return {}

    def get(self, endpoint: str):
        url = f"{self.baseurl}/{endpoint}"
        response = requests.get(url, headers=self._headers())
        return response.json()

    def put(self, endpoint: str, payload: dict):
        url = f"{self.baseurl}/{endpoint}"
        response = requests.put(url, json=payload, headers=self._headers())
        response.raise_for_status()
        return response.json()


import typer
from acemist.state import State
from acemist.context import CLIContext
from acemist.mist.mist import Mist
from acemist.commands import orgs, sites, context, networktemplates,device
from acemist.commands._self import self_cmd

app = typer.Typer()


@app.callback()
def main(ctx: typer.Context):
    cli_context = CLIContext()
    active = cli_context.get_active_context()
    ctx.obj = State(
        mist=Mist(
            baseurl=active.get("apiurl") if active else None,
            token=active.get("api_token") if active else None,
            org_id=active.get("org_id") if active else None,
            site_id=active.get("site_id") if active else None
        ),
        context=cli_context
    )


app.command("self")(self_cmd)
app.add_typer(context.app, name="context")
app.add_typer(sites.app, name="site")
app.add_typer(orgs.app, name="org")
app.add_typer(networktemplates.app, name="network_templates")
app.add_typer(networktemplates.app, name="nt")
app.add_typer(device.app, name="device")


def cli():
    app()


if __name__ == "__main__":
    cli()






import typer
from typing import Optional
from acemist.mist.mist import Mist, Region
from acemist.rest.rest import RestClient

app = typer.Typer(help="Context management. Configure API url etc.")

@app.command()
def add(
    name: str = typer.Option(..., "--name", "-n"),
    region: Region = typer.Option(..., "--region", "-r"),
    api_token: Optional[str] = typer.Option(None, "--token", "-t"),
    org_id: Optional[str] = typer.Option(None, "--org-id", "-o"),
    site_id:str = typer.Option(..., "--site-id", "-s"),
    ctx: typer.Context = typer.Context
):
    """Set or update a context."""
    cm = ctx.obj.context

    if api_token is None:
        api_token = typer.prompt("API token", hide_input=True)

    cm.set_context(name, region, api_token, org_id=org_id, site_id=site_id)

    # Auto-populate org from /self if org_id not provided
    if not org_id:
        try:
            mist = ctx.obj.mist
            mist._poplulate_self()
            data = mist.user_data
            orgs = [p for p in data.get("privileges", []) if p.get("scope") == "org"]
            if len(orgs) == 1:
                cm.update_org(orgs[0]["org_id"], orgs[0]["name"])
                typer.echo(f"Org auto-set to '{orgs[0]['name']}'")
            elif len(orgs) > 1:
                typer.echo("Multiple orgs found. Set one with: acemist context set-org --org-id <id>")
                for o in orgs:
                    typer.echo(f"  {o['org_id']}  {o['name']}")
        except Exception as e:
            typer.echo(f"Could not auto-detect org: {e}", err=True)

    typer.echo(f"Context '{name}' set and activated!")

@app.command()
def list(ctx: typer.Context):
    """List all saved contexts."""
    cm = ctx.obj.context
    contexts = cm.data.get('contexts', {})

    active = cm.data.get("active_context")
    if not contexts:
        typer.echo("No contexts saved.")
        return
    for name, data in contexts.items():
        marker = "*" if name == active else " "

        region = data.get("region", "")
        apiurl = data.get("apiurl", "")
        site_id= data.get("site_id","")
        typer.echo(f"{marker} {name}@{region} ({apiurl}) ({site_id})")

@app.command()
def use(name: str, ctx: typer.Context):
    """Set active context by name."""
    ctx.obj.set_active(name)
    typer.echo(f"Active context set to '{name}'")



import typer
from acemist.print_utils.print_sites import print_list_sites, print_site_details

app = typer.Typer(help="Manage sites.")


@app.command("list")
def list_sites(ctx: typer.Context):
    """List all sites."""
    mist = ctx.obj.mist
    site_data = mist.site_list()
    print_list_sites(site_data)


@app.command("show")
def get_site(
    ctx: typer.Context,
    
):
    """Get details for a specific site."""
    mist = ctx.obj.mist
    site_data = mist.site_get()

    print_site_details(site_data)




from rich.table import Table
from rich.console import Console
from rich import box
from datetime import datetime


def _fmt_time(ts):
    return datetime.fromtimestamp(ts).strftime("%Y-%m-%d") if ts else "-"


def print_list_network_templates(templates: list):
    console = Console()

    table = Table(box=box.ROUNDED, padding=(0, 1))
    table.add_column("Name", style="bold")
    table.add_column("Networks", justify="right")
    table.add_column("Port profiles", justify="right")
    table.add_column("Modified")
    table.add_column("ID", style="dim")

    for t in templates:
        networks = len(t.get("networks", {}))
        port_usages = len(t.get("port_usages", {}))
        table.add_row(
            t.get("name", "").strip(),
            str(networks) if networks else "-",
            str(port_usages) if port_usages else "-",
            _fmt_time(t.get("modified_time")),
            t.get("id", ""),
        )

    console.print(table)


def print_network_template_details(template: dict):
    console = Console()

    info = Table(box=box.ROUNDED, show_header=False, padding=(0, 1))
    info.add_column(style="bold cyan")
    info.add_column()

    info.add_row("Name", template.get('name'))
    info.add_row("Created", _fmt_time(template.get('created_time')))
    info.add_row("Modified", _fmt_time(template.get('modified_time')))
    info.add_row("Additional CMDs", str(len(template.get('additional_config_cmds'))))
    info.add_row("Networks", str(len(template.get('networks'))))
    info.add_row("VRFs", str(len(template.get('vrf_instances'))))
    info.add_row("Group Tags", str(len(template.get('group_tags'))))
    info.add_row("Import Org Nets", str(len(template.get('import_org_networks'))))
    console.print(info)


def print_network_template_nets(template: dict, highlighted_rows: list = []):

        console = Console()
        dim_others = bool(highlighted_rows)
        
        networks = template.get("networks", {})
        
        table = Table(
            title="Networks",
            box=box.ROUNDED,
            padding=(0, 1),
            show_lines=False,
        )

        table.add_column("Name", style="bold cyan", overflow="fold")
        table.add_column("VLAN", justify="right", style="magenta")
        table.add_column("Vrf", style="orange3")
        table.add_column("Subnet", style="green")
        table.add_column("Subnet6", style="green")

        for name, net in sorted(networks.items()):
            vlan = str(net.get("vlan_id", ""))
            subnet = net.get("subnet") or "-"
            subnet6 = net.get("subnet6") or "-"
            vrf = net.get("vrf") or "-"
            highlighted = name in highlighted_rows
            if highlighted:
                style = "bold green"
            elif dim_others:
                style = "dim"
            else:
                style = ""

            table.add_row(name, vlan, vrf,subnet, subnet6, style=style)
        
        console.print(table)


from rich.table import Table
from rich.console import Console
from rich import box


def print_self(data: dict):
    console = Console()

    info = Table(box=box.ROUNDED, show_header=False, padding=(0, 1))
    info.add_column(style="bold cyan")
    info.add_column()

    name = f"{data.get('first_name', '')} {data.get('last_name', '')}".strip()
    info.add_row("Name", name)
    info.add_row("Email", data.get("email", ""))
    phone = data.get("phone") or data.get("phone2") or "-"
    info.add_row("Phone", phone)

    console.print(info)

    privileges = data.get("privileges", [])
    if privileges:
        priv_table = Table(title="Privileges", box=box.ROUNDED, padding=(0, 1))
        priv_table.add_column("Org", style="bold")
        priv_table.add_column("Role")
        priv_table.add_column("Scope")
        priv_table.add_column("Views")
        priv_table.add_column("OrgId")

        for p in privileges:
            views = ", ".join(p.get("views", [])) or "-"
            priv_table.add_row(
                p.get("name", ""),
                p.get("role", ""),
                p.get("scope", ""),
                views,
                p.get("org_id", "")
            )

        console.print(priv_table)


import json
from pathlib import Path

CONTEXT_FILE = Path.home() / ".acemist/.acemist_cli_context.json"

class CLIContext:
    def __init__(self):
        self.data = self._load()

    def _load(self):
        if CONTEXT_FILE.exists():
            with open(CONTEXT_FILE) as f:
                content = f.read().strip()
                if not content:
                    return {}
                return json.loads(content)
        return {}

    def save(self):
        CONTEXT_FILE.parent.mkdir(parents=True, exist_ok=True)
        with open(CONTEXT_FILE, "w") as f:
            json.dump(self.data, f, indent=4)

    def set_context(self, name: str, region, api_token: str, org_id: str | None = None, org_name: str | None = None, site_id: str|None=None):
        apiurl = f"https://{region.api_url}/api/v1"
        ctxs = self.data.setdefault("contexts", {})
        ctxs[name] = {
            "apiurl": apiurl,
            "region": region.value,
            "api_token": api_token,
            
        }
        
        if org_id:
            ctxs[name]["org_id"] = org_id
        if org_name:
            ctxs[name]["org_name"] = org_name
        if site_id:
            ctxs[name]["site_id"]=site_id
        self.data["active_context"] = name
        self.save()

    def update_org(self, org_id: str, org_name: str):
        """Set org_id and org_name on the active context."""
        ctx = self.get_active_context()
        if ctx is not None:
            ctx["org_id"] = org_id
            ctx["org_name"] = org_name
            self.save()

    def set_active(self, name: str):
        self.data["active_context"] = name
        self.save()

    def get_active_context(self):
        name = self.data.get("active_context")
        return self.data.get("contexts", {}).get(name)

    def get_active_url(self):
        ctx = self.get_active_context()
        return ctx.get("apiurl") if ctx else None

    def get_active_org_id(self) -> str | None:
        ctx = self.get_active_context()
        return ctx.get("org_id") if ctx else None




from enum import Enum
from acemist.rest.rest import RestClient
from acemist.print_utils.print_self import print_self

class Region(str, Enum):
    """Mist cloud regions. Value is the CLI slug, attributes hold the URLs."""

    def __new__(cls, value: str, api_url: str):
        obj = str.__new__(cls, value)
        obj._value_ = value
        obj.api_url = api_url
        return obj

    GLOBAL_01 = ("global-01", "api.mist.com")
    GLOBAL_02 = ("global-02", "api.gc1.mist.com")
    GLOBAL_03 = ("global-03", "api.ac2.mist.com")
    GLOBAL_04 = ("global-04", "api.gc2.mist.com")
    GLOBAL_05 = ("global-05", "api.gc4.mist.com")
    EMEA_01   = ("emea-01", "api.eu.mist.com")
    EMEA_02   = ("emea-02", "api.gc3.mist.com")
    EMEA_03   = ("emea-03", "api.ac6.mist.com")
    EMEA_04   = ("emea-04", "api.gc6.mist.com")
    APAC_01   = ("apac-01", "api.ac5.mist.com")
    APAC_02   = ("apac-02", "api.gc5.mist.com")
    APAC_03   = ("apac-03", "api.gc7.mist.com")



class Mist:

    def __init__(self, baseurl: str | None = None, token: str | None = None, org_id: str | None=None, site_id: str | None=None):
        self.rest = RestClient(
            baseurl=baseurl or "https://api.mist.com/api/v1",
            token=token,
        )
        self.user_data = {}
        self.org_id = org_id
        self.site_id =site_id
        
    def _poplulate_self(self):
        self.user_data = self.rest.get("self")
        print(self.user_data)
    def _self(self):
        self._poplulate_self()
        print_self(self.user_data)

    def site_list(self):
        data = self.rest.get(f"orgs/{self.org_id}/sites")
        return data

    def site_get(self,):
        data = self.rest.get(f"sites/{self.site_id}")
        return data

    def network_templates_list(self):
        data = self.rest.get(f"orgs/{self.org_id}/networktemplates")
        return data

    def network_templates_get(self, template_id: str):
        data = self.rest.get(f"orgs/{self.org_id}/networktemplates/{template_id}")
        return data

    def network_templates_update(self, template_id: str, payload: dict):
        data = self.rest.put(f"orgs/{self.org_id}/networktemplates/{template_id}", payload)
        return data
    
    def site_network_templates_get(self):
        
        data = self.rest.get(f"sites/{self.site_id}/networktemplates/derived?")
        
        return data
    def site_list_devices(self):
        data = self.rest.get(f"sites/{self.site_id}/devices?type=switch&limit=100&page=1")
        return data
    
    def site_devices_get(self,device_id:str):
        data = self.rest.get(f"sites/{self.site_id}/devices/{device_id}")
        return data


import typer
from rich.table import Table
from rich.console import Console
from rich import box

app = typer.Typer(help="Manage organizations.")

@app.command("list")
def list_orgs(ctx: typer.Context):
    """List orgs you have access to."""
    orgs = ctx.obj.context.get_orgs()
    active = ctx.obj.context.get_active_org()
    active_id = active["org_id"] if active else None

    if not orgs:
        typer.echo("No orgs found. Run 'acemist self' first to populate.")
        return

    table = Table(box=box.ROUNDED, padding=(0, 1))
    table.add_column("")
    table.add_column("Name", style="bold")
    table.add_column("Org ID", style="dim")

    for org in orgs:
        marker = "*" if org["org_id"] == active_id else " "
        table.add_row(marker, org["name"], org["org_id"])

    Console().print(table)

@app.command("use")
def use_org(org_id: str = typer.Argument(..., help="Org ID to activate"), ctx: typer.Context = typer.Context):
    """Set active org."""
    ctx.obj.context.set_active_org(org_id)
    typer.echo(f"Active org set to {org_id}")


from rich.table import Table
from rich.console import Console
from rich import box
from datetime import datetime




def print_device_list(device_list,highlighted_rows: list = []):
    console = Console()
    table = Table(
        title="Devices",
        box=box.ROUNDED,
        padding=(0, 1),
        show_lines=False,
    )

    table.add_column("Name", style="bold cyan", overflow="fold")
    table.add_column("Device id", justify="right", style="magenta")
    
    for device in device_list:
        
        name=device['name']
        id= device['id']
        table.add_row(name, id)
    console.print(table)

def print_device_config(ospf_data,highlighted_rows: list = []):
    console = Console()
    table = Table(
        title=f"OSPF networks for device: [bold cyan]{ospf_data["device_name"]}",
        box=box.ROUNDED,
        padding=(0, 1),
        show_lines=False,
    )
    
    table.add_column("Network", style="bold cyan", overflow="fold")
    table.add_column("Passive", justify="right", style="magenta")
    table.add_column("Interface type", justify="right", style="orange3")
    
    for name, data in ospf_data["networks"].items():
        passive = "[green]True[/green]" if data["passive"] else "[red]False[/red]"
        interface_type = data["interface_type"]

        table.add_row(name, passive, interface_type,)
    console.print(table)

def print_dhcp_scopes(scope_data):
    console = Console()
    table = Table(
        title=f"DHCP scopes for device: [bold cyan]{scope_data["device_name"]}",
        box=box.ROUNDED,
        padding=(0, 1),
        show_lines=False,
    )
    #print(scope_data)
    table.add_column("Network", style="bold cyan", overflow="fold")
    table.add_column("Start IP", justify="right", style="magenta")
    table.add_column("End IP", justify="right", style="orange3")
    table.add_column("Gateway", justify="right", style="green")
    table.add_column("DNS1", justify="right", style="deep_pink2")
    table.add_column("DNS2", justify="right", style="deep_pink2")

    for name, data in scope_data["networks"].items():
        ip_start = data["Start IP"]
        ip_end = data["End IP"]
        gateway = data["Gateway"]
        dns1= data["DNS"][0]
        dns2 = data["DNS"][1] if len(data["DNS"]) >= 2 else "-"
        

        table.add_row(name, ip_start, ip_end,gateway,dns1,dns2)
    console.print(table)
    
def print_vrrp_gropus(vrrp_data):
    console = Console()
    table = Table(
        title=f"VRRP Grops for: [bold cyan]{vrrp_data["device_name"]}",
        box=box.ROUNDED,
        padding=(0, 1),
        show_lines=False,
    )
    
    table.add_column("VRRP Group ID", style="bold cyan", overflow="fold")
    table.add_column("Network", justify="right", style="magenta")
    table.add_column("ip", justify="right", style="orange3")
    
    
    for vrrp in vrrp_data["vrrp_groups"]:
        table.add_row(
            vrrp["id"],
            vrrp["network"],
            vrrp["ip"]
        )
    console.print(table)
     
def print_access(access_data):
    console = Console()
    table = Table(
        title=f"Accessconfig for: [bold cyan]{access_data["device_name"]}",
        box=box.ROUNDED,
        padding=(0, 1),
        show_lines=False,
    )
    
    table.add_column("Port", style="bold cyan", overflow="fold")
    table.add_column("Usage", justify="right", style="magenta")
    
    
    
    for access in access_data["port_config"]:
        table.add_row(
            access["port"],
            access["usage"],
            
        )
        
    console.print(table)


import typer
from typing import Optional, Annotated
from rich.table import Table
from rich.console import Console
from acemist.print_utils.print_network_templates import print_network_template_nets

app = typer.Typer(help="Manage networks within a Network Template.")





@app.command("add")
def add_network(
    ctx: typer.Context,
    template_id: str = typer.Argument(..., help="Network Template ID"),
    name: Optional[str] = typer.Option(None, help="Name of the Network"),
    subnet: Optional[str] = typer.Option(None, help="Subnet (CIDR)"),
    subnet6: Optional[str] = typer.Option(None, help="Subnet6 (CIDR), optional"),
    vlanid: Optional[str] = typer.Option(None, help="VlanId"),
):
    """Add a network to a Network Template."""
    # Required fields — prompt if not provided
    if name is None:
        name = typer.prompt("Network Name")
    if subnet is None:
        subnet = typer.prompt("Subnet")
    if vlanid is None:
        vlanid = typer.prompt("Vlan Id")

    # Optional fields — prompt with empty default so user can press Enter to skip
    if subnet6 is None:
        subnet6 = typer.prompt("Subnet6 (leave empty to skip)", default="") or None

    mist = ctx.obj.mist

    # Fetch network template
    template = mist.network_templates_get(template_id)

    # edit network template, add net
    new_net = {
        "vlan_id": vlanid,
        "subnet": subnet,
        "subnet6": subnet6
    }

    # Merge payload
    if name in template["networks"].keys():
        print("Network with same name exists in this template already! Exiting.")
        exit()

    else:
        template["networks"][name] = new_net

    # Confirm before add
    console = Console()
    console.print("Template networks after change: ")
    print_network_template_nets(template, highlighted_rows=[name])

    confirm = typer.confirm(f"Are you sure you want to add the network {name} to template {template["name"]}?")
    if not confirm:
        exit()

    # Put updated network template
    mist.network_templates_update(template_id, template)

@app.command("list")
def list_networks_vrf(
    ctx: typer.Context,
    template_id: str = typer.Argument(..., help="Network Template ID"),
    #site_id: str = typer.Argument(..., help="Site ID"),
):
    """List networks for a specific Network Template with VRF."""
    mist = ctx.obj.mist
    template = mist.network_templates_get(template_id)
    sites = mist.site_network_templates_get()
    networks = template.get("networks", [])
    
    def get_vrf(vrf_instances, network_name):
        for vrf, data in vrf_instances.items():
            if network_name in data.get("networks", []):
                return vrf
        return None

    network_names = [
        n["name"] if isinstance(n, dict) else n
        for n in networks
    ]

    for network_name in network_names:
        vrf = None

        for site_template in sites:
            vrf_instances = site_template.get("vrf_instances", {})
            vrf = get_vrf(vrf_instances, network_name)

            if vrf:
                template["networks"][network_name]["vrf"] = vrf
                
                break

        if not vrf:
            template["networks"][network_name]["vrf"] = "-"
            
    
    
    
    print_network_template_nets(template)


import typer
from enum import Enum

from acemist.print_utils.print_device import print_device_list,print_device_config,print_dhcp_scopes,print_vrrp_gropus,print_access
class ShowConfig(str, Enum):
    ospf="ospf"
    dhcp_scopes="dhcp"
    vrrp="vrrp"
    access="access"
app = typer.Typer(help="Manage devices.")


@app.command("list_devices")
def list_devices(
    ctx: typer.Context,
    ):
    """List devices in site"""
    mist = ctx.obj.mist
    
    device_list = mist.site_list_devices()
    
    devices= []
    for device in device_list:
        device_name_id= {
            "name": device['name'],
            "id": device['id']
        }
        devices.append(device_name_id)
    print_device_list(devices)

@app.command("show_config")
def show_config(
    ctx: typer.Context,
    device_id: str = typer.Argument(..., help="Device ID"),
    section: ShowConfig =None
    ):
    """Show device config"""
    mist = ctx.obj.mist
    

    device = mist.site_devices_get(device_id)

    if section =="ospf":

        """Show device config"""
        
        
        try:
            ospf_data = {
            "device_name": device["name"],
            "networks": {}
            }

            for network, net_data in device["ospf_areas"]["0"]["networks"].items():
                ospf_data["networks"][network] = {
                    "interface_type": net_data["interface_type"] if not net_data["passive"] else "-",
                    "passive": net_data["passive"],
                }
            print_device_config(ospf_data)
        except:
            "No OSPF configured"
    if section == "dhcp":
        """list DHCP scopes configured on device"""
        try:
            scope_data = {
            "device_name": device["name"],
            "networks": {}
            }
            
            for network, net_data in device["dhcpd_config"].items():
                
                if "enabled" not in network:
                    scope_data["networks"][network] = {
                        "Start IP": net_data["ip_start"] ,
                        "End IP": net_data["ip_end"] ,
                        "Gateway": net_data["gateway"],
                        "DNS":net_data["dns_servers"]
                    }
                
            print_dhcp_scopes(scope_data)
        except:
            print("no dhcp scopes found")
    if section == "vrrp":
        try:
            vrrp_data = {
            "device_name": device["name"],
            "vrrp_groups": []
            }
            
            for group_id, group_data in device["vrrp_groups"].items():
                for network_name, net_data in group_data["networks"].items():
                    vrrp_data["vrrp_groups"].append({
                        "id": group_id,
                        "network": network_name,
                        "ip": net_data["ip"]
                    })
                     
            print_vrrp_gropus(vrrp_data)
        except:
            print("No vrrp configuration found")
    if section == "access":
        try:
            access_data = {
            "device_name": device["name"],
            "port_config": []
            }
            
            for port_id, port_data in device["port_config"].items():
                
                access_data["port_config"].append({
                    "port":port_id,
                    "usage":port_data["usage"]
                })

            print_access(access_data)
        except:
            print("No accessport configuration found")








import typer
from acemist.print_utils.print_network_templates import (
    print_list_network_templates,
    print_network_template_details,
)
from acemist.commands import networktemplates_networks


app = typer.Typer(help="Manage Network Templates.")
app.add_typer(networktemplates_networks.app, name="networks")


@app.command("list")
def list_templates(ctx: typer.Context):
    """List all sites."""
    mist = ctx.obj.mist
    templates = mist.network_templates_list()
    print_list_network_templates(templates)


@app.command("details")
def get_template_details(
    ctx: typer.Context,
    template_id: str = typer.Argument(..., help="Site ID")
):
    """Get details for a specific site."""
    mist = ctx.obj.mist
    template = mist.network_templates_get(template_id)
    print_network_template_details(template)





from rich.table import Table
from rich.console import Console
from rich import box


def print_list_sites(sites: list):
    console = Console()

    table = Table(box=box.ROUNDED, padding=(0, 1))
    table.add_column("Name", style="bold")
    table.add_column("Address")
    table.add_column("Timezone", style="dim")
    table.add_column("ID", style="dim")

    for site in sites:
        table.add_row(
            site.get("name", ""),
            site.get("address", ""),
            site.get("timezone", ""),
            site.get("id", ""),
        )

    console.print(table)

def print_site_details(site: dict):
    from datetime import datetime

    console = Console()

    def fmt_time(ts):
        return datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H:%M") if ts else "-"

    def fmt_val(v):
        if v is None:
            return "[dim]-[/dim]"
        return str(v)

    info = Table(box=box.ROUNDED, show_header=False, padding=(0, 1))
    info.add_column(style="bold cyan", min_width=20)
    info.add_column()

    latlng = site.get("latlng", {})

    info.add_row("Name",          site.get("name", ""))
    info.add_row("ID",            site.get("id", ""))
    info.add_row("Org ID",        site.get("org_id", ""))
    info.add_row("Address",       site.get("address", ""))
    info.add_row("Country",       site.get("country_code", ""))
    info.add_row("Timezone",      site.get("timezone", ""))
    info.add_row("UTC offset",    str(site.get("tzoffset", "")))
    info.add_row("Lat/Lng",       f"{latlng.get('lat', '')}, {latlng.get('lng', '')}")
    info.add_row("Notes",         site.get("notes") or "-")
    info.add_row("Created",       fmt_time(site.get("created_time")))
    info.add_row("Modified",      fmt_time(site.get("modified_time")))

    console.print(info)

    templates = Table(title="Templates", box=box.ROUNDED, show_header=False, padding=(0, 1))
    templates.add_column(style="bold cyan", min_width=20)
    templates.add_column()

    for key in ("networktemplate_id", "gatewaytemplate_id", "routertemplate_id",
                "rftemplate_id", "aptemplate_id", "sitetemplate_id",
                "alarmtemplate_id", "secpolicy_id"):
        templates.add_row(key.replace("_", " ").title(), fmt_val(site.get(key)))

    console.print(templates)
